@@ -369,6 +369,37 @@ void JSONTest::testEmptyObject()
 }
 
 
+void JSONTest::testEmptyPropertyName()
+{
+	std::string json = "{\"\": 42}";
+	Parser parser;
+	Var result;
+
+	try
+	{
+		result = parser.parse(json);
+	}
+	catch(JSONException& jsone)
+	{
+		std::cout << jsone.message() << std::endl;
+		assert(false);
+	}
+
+	assert(result.type() == typeid(Object::Ptr));
+
+	Object::Ptr object = result.extract<Object::Ptr>();
+	assert(object->size() == 1);
+
+	DynamicStruct ds = *object;
+	assert (ds.size() == 1);
+
+	const DynamicStruct& rds = *object;
+	assert (rds.size() == 1);
+	
+	assert (ds[""] == 42);
+}
+
+
 void JSONTest::testComplexObject()
 {
 	std::string json = 
@@ -1181,6 +1212,61 @@ void JSONTest::testPrintHandler()
 		"    ]\n"
 		"}"
 	);
+
+	json = 
+		"{"
+			"\"array\":"
+			"["
+				"{"
+					"\"key1\":"
+					"["
+						"1,2,3,"
+						"{"
+							"\"subkey\":"
+							"\"test\""
+						"}"
+					"]"
+				"},"
+				"{"
+					"\"key2\":"
+					"{"
+						"\"anotherSubKey\":"
+						"["
+							"1,"
+							"{"
+								"\"subSubKey\":"
+								"["
+									"4,5,6"
+								"]"
+							"}"
+						"]"
+					"}"
+				"}"
+			"]"
+		"}";
+
+
+	ostr.str("");
+	pHandler->setIndent(0);
+	parser.reset();
+	parser.parse(json);
+	assert (json == ostr.str());
+
+	json="[[\"a\"],[\"b\"],[[\"c\"],[\"d\"]]]";
+	ostr.str("");
+	pHandler->setIndent(0);
+	parser.reset();
+	parser.parse(json);
+    std::cout << ostr.str() << std::endl;
+	assert (json == ostr.str());
+
+	json="[{\"1\":\"one\",\"0\":[\"zero\",\"nil\"]}]";
+	ostr.str("");
+	pHandler->setIndent(0);
+	parser.reset();
+	parser.parse(json);
+	assert (json == ostr.str());
+
 }
 
 
@@ -1340,13 +1426,27 @@ void JSONTest::testStringify()
 
 void JSONTest::testStringifyPreserveOrder()
 {
-	Object jObj(true);
-	jObj.set("foo", 0);
-	jObj.set("bar", 0);
-	jObj.set("baz", 0);
+	Object presObj(true);
+	presObj.set("foo", 0);
+	presObj.set("bar", 0);
+	presObj.set("baz", 0);
 	std::stringstream ss;
-	jObj.stringify(ss);
+	presObj.stringify(ss);
 	assert(ss.str() == "{\"foo\":0,\"bar\":0,\"baz\":0}");
+	ss.str("");
+	Stringifier::stringify(presObj, ss);
+	assert(ss.str() == "{\"foo\":0,\"bar\":0,\"baz\":0}");
+
+	Object noPresObj;
+	noPresObj.set("foo", 0);
+	noPresObj.set("bar", 0);
+	noPresObj.set("baz", 0);
+	ss.str("");
+	noPresObj.stringify(ss);
+	assert(ss.str() == "{\"bar\":0,\"baz\":0,\"foo\":0}");
+	ss.str("");
+	Stringifier::stringify(noPresObj, ss);
+	assert(ss.str() == "{\"bar\":0,\"baz\":0,\"foo\":0}");
 
 	std::string json = "{ \"Simpsons\" : { \"husband\" : { \"name\" : \"Homer\" , \"age\" : 38 }, \"wife\" : { \"name\" : \"Marge\", \"age\" : 36 }, "
 						"\"children\" : [ \"Bart\", \"Lisa\", \"Maggie\" ], "
@@ -1753,6 +1853,7 @@ CppUnit::Test* JSONTest::suite()
 #endif
 	CppUnit_addTest(pSuite, JSONTest, testStringProperty);
 	CppUnit_addTest(pSuite, JSONTest, testEmptyObject);
+	CppUnit_addTest(pSuite, JSONTest, testEmptyPropertyName);
 	CppUnit_addTest(pSuite, JSONTest, testComplexObject);
 	CppUnit_addTest(pSuite, JSONTest, testDoubleProperty);
 	CppUnit_addTest(pSuite, JSONTest, testDouble2Property);
