@@ -7,8 +7,6 @@
 // Package: MongoDB
 // Module:  KillCursorsRequest
 //
-// Implementation of the KillCursorsRequest class.
-//
 // Copyright (c) 2012, Applied Informatics Software Engineering GmbH.
 // and Contributors.
 //
@@ -23,8 +21,8 @@ namespace Poco {
 namespace MongoDB {
 
 
-KillCursorsRequest::KillCursorsRequest() 
-	: RequestMessage(MessageHeader::KillCursors)
+KillCursorsRequest::KillCursorsRequest():
+	RequestMessage(MessageHeader::OP_KILL_CURSORS)
 {
 }
 
@@ -38,7 +36,7 @@ void KillCursorsRequest::buildRequest(BinaryWriter& writer)
 {
 	writer << 0; // 0 - reserved for future use
 	writer << _cursors.size();
-	for(std::vector<Int64>::iterator it = _cursors.begin(); it != _cursors.end(); ++it)
+	for (std::vector<Int64>::iterator it = _cursors.begin(); it != _cursors.end(); ++it)
 	{
 		writer << *it;
 	}		
